#!/usr/bin/env python
"""
_Information on streamers files by run_

T0Mon: this class is used in the run page [streamer file table]
"""

from T0.DAS.Database.Oracle.LongResultQuery import LongResultQuery 

class StreamersByRun(LongResultQuery):
 
   
   sql="""SELECT \
       streamer.streamer_id AS streamer_id, \
       streamer.lumi_id     AS lumi_id, \
       streamer.filesize    AS filesize, \
       streamer.events      AS events, \
       stream.name          AS stream, \
       streamer.lfn         AS lfn, \
       streamer.run_id      AS run_id \
    FROM   streamer \
    JOIN  stream ON  streamer.stream_id =  stream.id \
    WHERE   streamer.run_id = %s \
    ORDER BY \
       streamer.streamer_id DESC """
        
   sqlrun=":run"
   sqlmaxrun="(SELECT max(run_id) AS run FROM run  )"
   
   def execute(self, run,uplimit,downlimit, conn = None, transaction = False):

        bindVars={"uplimit":uplimit,"downlimit":downlimit}
        try:
          if int(run)>0:
           self.sql=self.sql % (self.sqlrun)
           bindVars.update({"run": run})
          else: 
            raise Exception('zeroRun')
        except Exception:
           self.sql=self.sql % (self.sqlmaxrun)
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        return self.formatDict(result) 
	  
########################################################
from WMCore.Database.DBFormatter import DBFormatter

class StreamersByRun_count(DBFormatter):
   sql="""SELECT COUNT(DISTINCT   streamer.streamer_id)
    FROM   streamer 
    JOIN  stream ON  streamer.stream_id =  stream.id 
    WHERE   streamer.run_id = %s """
   sqlrun=":run"
   sqlmaxrun="(SELECT max(run_id) AS run FROM run  )"
   
   def execute(self, run, conn = None, transaction = False):
        bindVars={}
        try:
          if int(run)>0:
           self.sql=self.sql % (self.sqlrun)
           bindVars.update({"run": run})
          else: 
            raise Exception('zeroRun')
        except Exception:
           self.sql=self.sql % (self.sqlmaxrun)
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        return self.format(result) 