#!/usr/bin/env python
"""
_ Express job Lifetimes_

T0Mon query: this class is used to produce the Express statistical plots 
"""

from WMCore.Database.DBFormatter import DBFormatter

class ExpressJobLifetime(DBFormatter):
    """
  Print stats on express job lifetime [startTime/endTime/difference]
  if run is not an integer or run<=0 then the statistics are the whole system's ones.
  if run>0 then the statistics are the single run's ones.

    """
    
    sql= """
     SELECT 
     DISTINCT j.job_id as job_id,
     j.definition_time as startTime, 
     j.completion_time as endTime,
     j.completion_time-j.definition_time as diff
     FROM express_job_def j    %s
         RIGHT OUTER JOIN job_status js  ON ( js.id =j.job_status ) 
         WHERE js.status='Success'  
           %s
     ORDER BY j.definition_time"""
         
    sqlRunExtension="""  
         INNER JOIN express_job_streamer_assoc assoc ON (j.job_id = assoc.job_id)  
         INNER JOIN ( 
               SELECT s.streamer_id from  streamer s 
               WHERE   s.run_id =:run  
               )stream ON (stream.streamer_id = assoc.streamer_id)"""
    sqlTimeLimits="""
           AND j.completion_time> :stime
           AND j.definition_time< :etime  """
    sqlNotNullCheck=" AND j.job_id>0 " #avoid null tuples
         
    def buildQuery(self):
       """
        This method [overloaded in the sub-classes] complete the SQL string adding the needed parts 
       """
       self.sql=self.sql%(self.sqlRunExtension,self.sqlTimeLimits)   
       
    def execute(self,stime,etime,run, conn = None, transaction = False):
        """
        Execute the SQL for the given Express statistics and then format and return
        the result.
        The parameters are unix timestamps (integer) with the extremes of the temporal window to monitor [sTime<eTime]
        """
        bindVars={}
        try:
          if len(stime)>0 and len(etime)>0:
            bindVars = {"stime": str(stime),"etime": str(etime)}
          else: 
            raise Exception('noTime')
        except Exception:
          self.sqlTimeLimits=self.sqlNotNullCheck
        try:
          #print run
          if int(run)>0:
            self.buildQuery()
            bindVars.update({"run": run})
          else: 
            raise Exception('zeroRun')
        except Exception:
           #print e
           self.sqlRunExtension=""
           self.buildQuery()
        #print self.sql
        #at least one of the 2 options (time/run) has to be activate:
        if bindVars=={}: 
          return {"Error" : "missing parameter(s)"}
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        return self.formatDict(result)
          
          
          
          
          
          