"""
API interface specific to ExpressInjector
"""

__revision__ = "$Id: ExpressInjectorAPI.py,v 1.4 2010/03/31 09:26:55 hufnagel Exp $"
__version__ = "$Revision: 1.4 $"

import logging

from T0.DataStructs.StreamerJobEntity import StreamerJobEntity

from T0.State.Database.Reader.ListStreamers import getAvailableExpressStreamers
from T0.State.Database.Writer.InsertStreamer import updateSplitStreamers

from T0.State.Database.Reader.ListJobs import getNextJobID
from T0.State.Database.Writer.InsertJob import insertJobDef
from T0.State.Database.Writer.InsertJob import assocExpressJobToStreamer


def defineJobs(dbConn, maxInputEvents):
    """
    _defineJobs_

    builds new express jobs

    usually one per streamer file, but if the streamers
    have few events, they can be combined up to maxInputEvents,
    but only within the same lumi section

    """
    streamers = getAvailableExpressStreamers(dbConn)

    streamerDict = {}
    for streamer in streamers:

        run = streamer['RUN_ID']
        stream = streamer['STREAM_ID']
        lumi = streamer['LUMI_ID']

        if streamerDict.has_key(run):
            if streamerDict[run].has_key(stream):
                if streamerDict[run][stream].has_key(lumi):
                    streamerDict[run][stream][lumi].append(streamer)
                else:
                    streamerDict[run][stream][lumi] = [ streamer ]
            else:
                streamerDict[run][stream] = { lumi : [ streamer ] }
        else:
            streamerDict[run] = { stream : { lumi : [ streamer ] } }

    jobEntityList = []
    for run_id in sorted(streamerDict.keys()):
        for stream_id in sorted(streamerDict[run_id].keys()):
            for lumi_id in sorted(streamerDict[run_id][stream_id].keys()):

                lumiStreamerList = streamerDict[run_id][stream_id][lumi_id]

                while len(lumiStreamerList) > 0:

                    eventsTotal = 0
                    streamerList = []

                    for streamer in lumiStreamerList:

                        # if first streamer, always use it
                        if len(streamerList) == 0:
                            eventsTotal = streamer['EVENTS']
                            streamerList.append(streamer)
                        # otherwise calculate new totals and check if to use streamer
                        else:
                            newEventsTotal = eventsTotal + streamer['EVENTS']

                            if newEventsTotal <= maxInputEvents:
                                eventsTotal = newEventsTotal
                                streamerList.append(streamer)

                    jobEntity = StreamerJobEntity( jobID = getNextJobID(dbConn,"Express"),
                                                   runNumber = run_id,
                                                   streamID = stream_id )
                    jobEntity.addStreamers(streamerList)
                    jobEntityList.append(jobEntity)

                    for streamer in streamerList:
                        lumiStreamerList.remove(streamer)

    return jobEntityList


def registerJobs(dbConn, jobEntityList):
    """
    _registerJobs_

    register list of exppress processing jobs in T0AST
    
    For each job
      create a new job definition & get ID from database
      update streamer status
      insert job to streamer association

    """
    streamerIDs = []
    for jobEntity in jobEntityList:

        insertJobDef(dbConn, "Express",
                     "Used", jobEntity['jobID'])
        assocExpressJobToStreamer(dbConn, jobEntity['jobID'],
                                  jobEntity['streamerIDs'])
        streamerIDs.extend(jobEntity['streamerIDs'])

    updateSplitStreamers(dbConn, streamerIDs)
