#!/usr/bin/env python
"""
_ListFilesForExpressMerge_

Oracle implementation of ListFilesForExpressMerge.

Return a list of files that are available for express merging.
This is used for ExpressMerge in the T0.
"""

__revision__ = "$Id: ListFilesForExpressMerge.py,v 1.3 2010/07/20 09:35:45 hufnagel Exp $"
__version__ = "$Revision: 1.3 $"

from WMCore.Database.DBFormatter import DBFormatter

class ListFilesForExpressMerge(DBFormatter):

    def execute(self, subscription = None, conn = None, transaction = False):

        mergeableFiles = []

        sql = """SELECT wmbs_fileset_files.fileid AS id,
                        express_file_info.stream_id AS stream_id,
                        express_file_info.primary_dataset AS primds_id,
                        express_file_info.processed_dataset AS procds_id,
                        data_tier.name AS datatier,
                        wmbs_file_runlumi_map.run AS run,
                        wmbs_file_runlumi_map.lumi AS lumi,
                        wmbs_file_details.filesize AS filesize,
                        lumi_section_express_done.insert_time AS lumi_done_time
                 FROM wmbs_fileset_files
                 INNER JOIN wmbs_file_details ON
                   wmbs_file_details.id = wmbs_fileset_files.fileid
                 INNER JOIN express_file_info ON
                   express_file_info.file_id = wmbs_fileset_files.fileid
                 INNER join data_tier ON
                   express_file_info.data_tier = data_tier.id
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_fileset_files.fileid
                 LEFT OUTER JOIN wmbs_sub_files_acquired ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_acquired.fileid AND
                   wmbs_sub_files_acquired.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_complete ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_complete.fileid AND
                   wmbs_sub_files_complete.subscription = %d
                 LEFT OUTER JOIN wmbs_sub_files_failed ON
                   wmbs_fileset_files.fileid = wmbs_sub_files_failed.fileid AND
                   wmbs_sub_files_failed.subscription = %d
                 INNER JOIN lumi_section_express_done ON
                   lumi_section_express_done.run_id = wmbs_file_runlumi_map.run AND
                   lumi_section_express_done.lumi_id = wmbs_file_runlumi_map.lumi AND
                   lumi_section_express_done.stream_id = express_file_info.stream_id 
                 WHERE wmbs_fileset_files.fileset = (SELECT fileset FROM wmbs_subscription WHERE id = %d)
                 AND wmbs_sub_files_acquired.fileid IS NULL
                 AND wmbs_sub_files_complete.fileid IS NULL
                 AND wmbs_sub_files_failed.fileid IS NULL
                 """ % (subscription,
                        subscription,
                        subscription,
                        subscription)

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)
        mergeableFiles.extend(self.formatDict(results))

        return mergeableFiles
