#!/usr/bin/env python
"""
_ListFilesForMerge_

Oracle implementation of ListFilesForRepack.

Return a list of files that are available for repacking.
This is used for Repacking in the T0.
"""

__revision__ = "$Id: ListFilesForRepack.py,v 1.1 2010/08/03 09:09:25 hufnagel Exp $"
__version__ = "$Revision: 1.1 $"

from WMCore.Database.DBFormatter import DBFormatter

import logging

class ListFilesForRepack(DBFormatter):

    def execute(self, conn = None, transaction = False):

        sql = """SELECT run_id FROM run
                 WHERE run_status = (SELECT id FROM run_status WHERE status = 'Active')"""

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)

        activeRuns = []
        for result in self.formatDict(results):
            activeRuns.append(result['run_id'])

        sql = """SELECT streamer.streamer_id AS id,
                        streamer.run_id AS run,
                        streamer.lumi_id AS lumi,
                        streamer.events AS events,
                        streamer.filesize AS filesize,
                        streamer.stream_id AS stream_id
                  FROM streamer
                  INNER JOIN run_stream_style_assoc ON
                    run_stream_style_assoc.run_id = streamer.run_id AND
                    run_stream_style_assoc.stream_id = streamer.stream_id AND
                    run_stream_style_assoc.style_id = (SELECT id FROM processing_style WHERE name = 'Bulk')
                  INNER JOIN lumi_section_closed ON
                    lumi_section_closed.run_id = streamer.run_id AND
                    lumi_section_closed.lumi_id = streamer.lumi_id AND
                    lumi_section_closed.stream_id = streamer.stream_id AND
                    lumi_section_closed.close_time > 0
                  WHERE checkForZeroState(streamer.used) = 0
                  """

        results = self.dbi.processData(sql, {}, conn = conn,
                                       transaction = transaction)

        #
        # sort by run and stream_id, also add closeout state
        #
        streamerDict = {}
        for result in self.formatDict(results):

            run = result['run']
            stream_id = result['stream_id']
            lumi = result['lumi']

            if not streamerDict.has_key(run):
                streamerDict[run] = { 'closeout' : False, 'streams' : {} }

            if not streamerDict[run]['streams'].has_key(stream_id):
                streamerDict[run]['streams'][stream_id] = {}

            if not streamerDict[run]['streams'][stream_id].has_key(lumi):
                streamerDict[run]['streams'][stream_id][lumi] = []

            streamerDict[run]['streams'][stream_id][lumi].append(result)

        # now checks for run in closeout
        for run in streamerDict.keys():
            if run not in activeRuns:
                streamerDict[run]['closeout'] = True

        return streamerDict
