#!/usr/bin/env python
"""
_Merge_

Job handler for Merge jobs.
"""

__revision__ = "$Id: Merge.py,v 1.30 2010/08/03 22:52:07 hufnagel Exp $"
__version__ = "$Revision: 1.30 $"

import logging
from sets import Set

from WMCore.WMBS.File import File as WMBSFile
from WMCore.WMBS.Fileset import Fileset as WMBSFileset

from T0.State.Database.Reader import ListFiles
from T0.State.Database.Writer import InsertRepacked
from T0.Tier0Accountant.WMBSBaseHandler import WMBSBaseHandler

class Merge(WMBSBaseHandler):
    """
    _RepackMerge_

    Job handler for Merge jobs.
    """
    
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        Given a file that was returned by a merge job, insert it into T0AST and
        setup all the necessary associations.
        """
        logging.debug("MergeHandler handles successful job ID:%s"  
                      % jobInfo["JOB_IDENTIFIER"])
        
        if len(jobFiles) != 1:
            logging.error("Got back too many merged files: %s" %
                          len(jobFiles))
            return
        #only one out put file for merge but in case more than one file output will allowed
        # update merge flag
        for jobFile in jobFiles:
            jobFile["MERGED"] = True
            
        wmbsJob = WMBSBaseHandler.handleSuccessfulJob(self, jobInfo, jobFiles)
        
        if wmbsJob == None:
            # error message handled by base class
            return
        
        return
    
    
    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list depending on t0astFile for next processing
        The list will depends on various factor (tier, merge by size conditon)
        If the file can be added in the fileset one at a time without waiting for 
        the job group to be finished, use this list
        """
        
        # check the data tier for the merged file
        #
        # all tiers go into DBSUploadable and WMBSPublishable
        # RAW also goes into Reconstructable
        # DQM also goes into DQMenable

        filesetList = []

        filesetList.append(WMBSFileset(name = "DBSUploadable"))

        if t0astFile["DATA_TIER"] == "RAW":
            filesetList.append(WMBSFileset(name = "Reconstructable"))
        elif t0astFile["DATA_TIER"] == "DQM":
            filesetList.append(WMBSFileset(name = "DQMenable"))
        elif t0astFile["DATA_TIER"] == "ALCAPROMPT":
            filesetList.append(WMBSFileset(name = "AlcaHarvestable"))

        return filesetList
